beta = 0.25;
gamma = 0.1;
y0 = [0.999, 0.001, 0];
h = 0.1;
t0 = 0;
tmax = 100;

f = @(t, y) covidModel(y, beta, gamma);

[t, y] = RK2(f, h, y0, t0, tmax);

figure(1);
clf;
plot(t, y(:,2));
xlabel('Time [days]');
ylabel('Infected fraction I(t) (beta=0.25)');

%[t1,y1] = ode23(@(t, y) covidModel(y, beta, gamma)', [t0, tmax], y0');

% test ratio checking at t = 50
fprintf("%.3f    ", testRatio(@(h) RK2(f, h, y0, t0, tmax), h, 50, 7))
fprintf("\n");


% Curve flattening for beta = 0.5, 0.4, 0.3, 0.2
figure(2);
clf;
hold on;
[tt1, yy1] = RK2(@(t, y) covidModel(y, 0.4, gamma), h, y0, t0, tmax);
[tt2, yy2] = RK2(@(t, y) covidModel(y, 0.3, gamma), h, y0, t0, tmax);
[tt3, yy3] = RK2(@(t, y) covidModel(y, 0.2, gamma), h, y0, t0, tmax);
[tt4, yy4] = RK2(@(t, y) covidModel(y, 0.1, gamma), h, y0, t0, tmax);
plot(tt1,yy1(:,2), tt2,yy2(:,2), tt3,yy3(:,2), tt4,yy4(:,2));
xlabel('Time [days]');
ylabel('Infected fraction I(t)');
legend('beta = 0.4', 'beta = 0.3', 'beta = 0.2', 'beta = 0.1');

figure(3);
clf;
beta0 = 0.15;
beta1 = 0.4;
trel = 120 + 1/pi;
tmax1 = 200;


frel = @(t, y) covidModel(y, beta0 + heaviside(t - trel) * (beta1-beta0), gamma);
[tt, yy] = RK2(frel, h, y0, t0, tmax1);
plot(tt,yy(:,2));

meth = @(h) RK2(@(t, y) covidModel(y, beta0 + heaviside(t - trel) * (beta1-beta0), gamma), h, y0, t0, tmax1);

format shortg

fprintf("%.3f    ",testRatio(meth, h, 100, 7))
fprintf("\n");
fprintf("%.3f    ",testRatio(meth, h, 150, 7))
fprintf("\n");

% There is the convergence order loss. The breaking point trel acts in a way
% that original IVP divides into two IVPs. The second one is defined from
% t>trel and with the initial y(trel) and beta = beta1. The RK2 method
% misses to compute y(trel), but computes y(trel'), where trel' is the next
% closest grid point to trel (i.e. trel' = t0 + i*h) and continues the
% computation of the second IVP from trel' with an error in the initial
% value y(trel'). It propagates further (Butterfly effect) and hence
% disturbs the convergence order. That's why the ratio conditions are far
% from 4. However, by decreasing the step size, at one point, this error
% becomes sufficiently small, so the convergence order might be restored.

function [t,y] = RK2(f, h, y0, t0, tmax)
%  [t,y] = ode23(@(t,y)f(t,y)', t0:h:tmax, y0');


    t = t0:h:tmax;
    n = length(y0);
    nsteps = floor((tmax-t0)/h)+1;
    
    y = zeros(nsteps, n);
    
    y(1,:) = y0;
    
    for i=2:nsteps
        K1 = h * f(t(i-1),y(i-1,:));
        K2 = h * f(t(i-1)+h/2, y(i-1,:) + K1/2);
        y(i,:) = y(i-1,:) + K2;
    end
end

function dx = covidModel(x, beta, gamma)
    s = x(1); i = x(2); r = x(3);
    ds = -beta*s*i;
    di = beta*s*i - gamma*i;
    dr = gamma * i;
    dx = [ds, di, dr];
end



